@@ -1,7 +1,7 @@
 /*
  *  Copyright © 1998, 2003 Jonathan Blandford <jrb@alum.mit.edu>
  *  Copyright © 2003 Callum McKenzie <callum@physics.otago.ac.nz>
- *  Copyright © 2007, 2008 Christian Persch
+ *  Copyright © 2007, 2008, 2009 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,11 +38,15 @@
 #endif
 #endif /* HAVE_HILDON */
 
+#include <libgames-support/games-card-theme.h>
+#include <libgames-support/games-card-themes.h>
 #include <libgames-support/games-clock.h>
+#include <libgames-support/games-debug.h>
 #include <libgames-support/games-files.h>
 #include <libgames-support/games-stock.h>
 #include <libgames-support/games-runtime.h>
 #include <libgames-support/games-sound.h>
+#include <libgames-support/games-string-utils.h>
 
 #include "board.h"
 #include "conf.h"
@@ -60,7 +64,11 @@
 #define MAIN_MENU_UI_PATH       "/MainMenu"
 #define RECENT_GAMES_MENU_PATH  MAIN_MENU_UI_PATH "/GameMenu/RecentMenu"
 #define OPTIONS_MENU_PATH       MAIN_MENU_UI_PATH "/OptionsMenu"
+#ifdef HAVE_HILDON
 #define CARD_THEMES_MENU_PATH   MAIN_MENU_UI_PATH "/ViewMenu/ThemeMenu"
+#else
+#define CARD_THEMES_MENU_PATH   MAIN_MENU_UI_PATH "/ViewMenu/ThemeMenu/ThemesPH"
+#endif
 #define TOOLBAR_UI_PATH         "/Toolbar"
 
 /* The maximum number of recent games saved */
@@ -95,6 +103,9 @@ struct _AisleriotWindowPrivate
   AisleriotGame *game;
   AisleriotBoard *board;
 
+  GamesCardThemes *theme_manager;
+  GamesCardTheme *theme;
+
 #ifdef HAVE_HILDON
   GtkLabel *game_message_label;
   guint game_message_hash;
@@ -133,7 +144,6 @@ struct _AisleriotWindowPrivate
 
   guint load_idle_id;
 
-  guint scalable_cards : 1;
   guint use_pixbuf_drawing : 1;
   guint changing_game_type : 1;
   guint freecell_mode : 1;
@@ -234,7 +244,7 @@ select_game_cb (GtkAction *action,
         strcmp (game_file, "sol.scm") == 0)
       continue;
 
-    game_name = aisleriot_util_get_display_filename (game_file);
+    game_name = games_filename_to_display_name (game_file);
 
     gtk_list_store_insert_with_values (GTK_LIST_STORE (list), &iter,
                                        -1,
@@ -606,7 +616,7 @@ help_about_cb (GtkAction *action,
 			     "AisleRiot is a part of GNOME Games."),
 #endif
                          "copyright", "Copyright © 1998-2006 Jonathan Blandford\n"
-                                      "Copyright © 2007, 2008 Christian Persch",
+                                      "Copyright © 2007, 2008, 2009 Christian Persch",
                          "license", licence,
                          "authors", authors,
                          "artists", artists,
@@ -679,6 +689,21 @@ statistics_cb (GtkAction *action,
   gtk_window_present (GTK_WINDOW (priv->stats_dialog));
 }
 
+#ifdef ENABLE_CARD_THEMES_INSTALLER
+
+static void
+install_themes_cb (GtkAction *action,
+                   AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+
+  games_card_themes_install_themes (priv->theme_manager,
+                                    GTK_WINDOW (window),
+                                    gtk_get_current_event_time ());
+}
+
+#endif /* ENABLE_CARD_THEMES_INSTALLER */
+
 #ifdef ENABLE_DEBUG_UI
 
 #ifndef HAVE_HILDON
@@ -716,7 +741,12 @@ static void
 delayed_move_to_next_screen_cb (GtkAction *action,
                                 GtkWidget *widget)
 {
+#if GLIB_CHECK_VERSION (2, 14, 0)
+  g_timeout_add_seconds (10, (GSourceFunc) delayed_move_to_next_screen_timeout_cb, widget);
+#else
   g_timeout_add (10 * 1000, (GSourceFunc) delayed_move_to_next_screen_timeout_cb, widget);
+#endif /* GLIB 2.14.0 */
+
 }
 
 static void
@@ -1010,6 +1040,11 @@ static void
 fullscreen_toggled_cb (GtkToggleAction *action,
                        GtkWindow *window)
 {
+  _games_debug_print (GAMES_DEBUG_WINDOW_STATE,
+                      "[window %p] fullscreen_toggled_cb, %s fullscreen\n",
+                      window,
+                      gtk_toggle_action_get_active (action) ? "going" : "leaving");
+
   if (gtk_toggle_action_get_active (action)) {
     gtk_window_fullscreen (window);
   } else {
@@ -1056,6 +1091,24 @@ sound_toggle_cb (GtkToggleAction *action,
   games_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_SOUND), sound_enabled);
 }
 
+#ifdef HAVE_CLUTTER
+
+static void
+animations_toggle_cb (GtkToggleAction *action,
+                      AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+  gboolean enabled;
+
+  enabled = gtk_toggle_action_get_active (action);
+
+  aisleriot_board_set_animation_mode (priv->board, enabled);
+  
+  games_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_ANIMATIONS), enabled);
+}
+
+#endif /* HAVE_CLUTTER */
+
 static void
 show_hint_cb (GtkAction *action,
               AisleriotWindow *window)
@@ -1240,9 +1293,6 @@ install_options_menu (AisleriotWindow *window)
     GtkToggleAction *action;
     gchar actionname[32];
 
-    g_print ("installing option value=%08x set=%d type=%d name='%s'\n",
-             option->value, option->set, option->type, option->display_name);
-
     g_snprintf (actionname, sizeof (actionname), "Option%u", option->value);
 
     if (option->type == AISLERIOT_GAME_OPTION_CHECK) {
@@ -1380,7 +1430,7 @@ install_recently_played_menu (AisleriotWindow *window)
     char *game_name, *tooltip;
 
     g_snprintf (actionname, sizeof (actionname), "Recent%d", i);
-    game_name = aisleriot_util_get_display_filename (recent_games[i]);
+    game_name = games_filename_to_display_name (recent_games[i]);
 #ifdef HAVE_HILDON
     tooltip = NULL;
 #else
@@ -1410,41 +1460,99 @@ install_recently_played_menu (AisleriotWindow *window)
 
 /* Card Theme menu */
 
+static void
+aisleriot_window_take_card_theme (AisleriotWindow *window,
+                                  GamesCardTheme *theme /* adopting */)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+  GtkWidget *widget = GTK_WIDGET (window);
+
+  if (theme == priv->theme)
+    return;
+
+  if (priv->theme) {
+    g_object_unref (priv->theme);
+  }
+  priv->theme = theme;
+
+  if (gtk_widget_has_screen (widget)) {
+    const cairo_font_options_t *font_options;
+
+    font_options = gdk_screen_get_font_options (gtk_widget_get_screen (widget));
+    games_card_theme_set_font_options (theme, font_options);
+  }
+
+  aisleriot_board_set_card_theme (priv->board, theme);
+}    
+
 static void
 card_theme_changed_cb (GtkToggleAction *action,
                        AisleriotWindow *window)
 {
   AisleriotWindowPrivate *priv = window->priv;
-  const char *theme;
+  GamesCardThemeInfo *current_theme_info = NULL, *new_theme_info;
+  GamesCardTheme *theme;
+  const char *theme_name;
 
   if (!gtk_toggle_action_get_active (action))
     return;
 
-  theme = g_object_get_data (G_OBJECT (action), "card-theme");
-  g_return_if_fail (theme != NULL);
+  new_theme_info = g_object_get_data (G_OBJECT (action), "theme-info");
+  g_assert (new_theme_info != NULL);
 
-#ifdef HAVE_GNOME
-  /* Compatibility with old settings */
-  if (g_str_has_suffix (theme, ".svg")) {
-    *g_strrstr (theme, ".svg") = '\0';
-  } else if (g_str_has_suffix (theme, ".png")) {
-    *g_strrstr (theme, ".png") = '\0';
+  if (priv->theme) {
+    current_theme_info = games_card_theme_get_theme_info (priv->theme);
   }
-#endif /* HAVE_GNOME */
 
-  if (aisleriot_board_set_card_theme (priv->board, theme)) {
-    games_conf_set_string (NULL, aisleriot_conf_get_key (CONF_THEME), theme);
+  if (games_card_theme_info_equal (new_theme_info, current_theme_info))
+    return;
+
+  theme = games_card_themes_get_theme (priv->theme_manager, new_theme_info);
+  if (!theme) {
+    GSList *group, *l;
+
+#if GTK_CHECK_VERSION (2, 12, 0) || (defined (HAVE_HILDON) && !defined(HAVE_MAEMO_3))
+    gtk_widget_error_bell (GTK_WIDGET (window));
+#endif
+
+    /* Set this action insensitive so we don't try again */
+    gtk_action_set_sensitive (GTK_ACTION (action), FALSE);
+
+    /* Re-set the radio action of the current theme to active */
+    group = gtk_radio_action_get_group (GTK_RADIO_ACTION (action));
+    for (l = group; l != NULL; l = l->next) {
+      GtkToggleAction *theme_action = GTK_TOGGLE_ACTION (l->data);
+      GamesCardThemeInfo *info;
+
+      if (theme_action == action)
+        continue;
+
+      info = g_object_get_data (G_OBJECT (theme_action), "theme-info");
+      if (!games_card_theme_info_equal (info, current_theme_info))
+        continue;
+
+      /* The check at the top will prevent an infinite loop */
+      gtk_toggle_action_set_active (theme_action, TRUE);
+      break;
+    }
+
+    return;
   }
+
+  aisleriot_window_take_card_theme (window, theme);
+
+  theme_name = games_card_theme_info_get_persistent_name (new_theme_info);
+  games_conf_set_string (NULL, aisleriot_conf_get_key (CONF_THEME), theme_name);
 }
 
 static void
-install_card_theme_menu (AisleriotWindow *window)
+install_card_theme_menu (GamesCardThemes *theme_manager,
+                         AisleriotWindow *window)
 {
   AisleriotWindowPrivate *priv = window->priv;
-  GamesFileList *themes;
-  GList *l;
+  GList *list, *l;
   GSList *radio_group = NULL;
-  const char *card_theme;
+  GamesCardThemeInfo *current_theme_info;
   guint i = 0;
 
   /* Clean out the old menu */
@@ -1460,11 +1568,12 @@ install_card_theme_menu (AisleriotWindow *window)
   /* See gtk bug #424448 */
   gtk_ui_manager_ensure_update (priv->ui_manager);
 
-  themes = games_file_list_card_themes (priv->scalable_cards);
+  list = games_card_themes_get_themes (priv->theme_manager);
 
   /* No need to install the menu when there's only one theme available anyway */
-  if (themes->list == NULL || themes->list->next == NULL) {
-    g_object_unref (themes);
+  if (list == NULL || list->next == NULL) {
+    g_list_foreach (list, (GFunc) games_card_theme_info_unref, NULL);
+    g_list_free (list);
     return;
   }
 
@@ -1474,15 +1583,19 @@ install_card_theme_menu (AisleriotWindow *window)
 
   priv->card_themes_merge_id = gtk_ui_manager_new_merge_id (priv->ui_manager);
 
-  card_theme = aisleriot_board_get_card_theme (priv->board);
+  if (priv->theme) {
+    current_theme_info = games_card_theme_get_theme_info (priv->theme);
+  } else {
+    current_theme_info = NULL;
+  }
 
-  for (l = themes->list; l != NULL; l = l->next) {
-    const char *theme = (const char *) l->data;
+  for (l = list; l != NULL; l = l->next) {
+    GamesCardThemeInfo *info = (GamesCardThemeInfo *) l->data;
     GtkRadioAction *action;
     char actionname[32];
     char *display_name, *tooltip;
 
-    display_name = aisleriot_util_get_display_filename (theme);
+    display_name = g_strdup (games_card_theme_info_get_display_name (info));
 
     g_snprintf (actionname, sizeof (actionname), "Theme%d", i);
 #ifdef HAVE_HILDON
@@ -1498,13 +1611,13 @@ install_card_theme_menu (AisleriotWindow *window)
     radio_group = gtk_radio_action_get_group (action);
 
     /* Check if this is the current theme's action. Do this before connecting the callback */
-    if (strcmp (card_theme, theme) == 0) {
+    if (games_card_theme_info_equal (info, current_theme_info)) {
       gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
     }
 
-    /* We steal the data from the list. A bit hackish, but works fine for now */
-    g_object_set_data_full (G_OBJECT (action), "card-theme",
-                            l->data, (GDestroyNotify) g_free);
+    /* We steal the data from the list */
+    g_object_set_data_full (G_OBJECT (action), "theme-info",
+                            l->data, (GDestroyNotify) games_card_theme_info_unref);
     l->data = NULL;
 
     g_signal_connect (action, "toggled",
@@ -1521,7 +1634,26 @@ install_card_theme_menu (AisleriotWindow *window)
     ++i;
   }
 
-  g_object_unref (themes);
+  /* The list elements' data's refcount has been adopted above */
+  g_list_free (list);
+}
+
+#ifdef HAVE_HILDON
+static void
+main_menu_show_cb (GtkMenu *menu,
+                   AisleriotWindow *window)
+#else
+static void
+view_menu_activate_cb (GtkAction *action,
+                       AisleriotWindow *window)
+#endif    
+{
+  AisleriotWindowPrivate *priv = window->priv;
+
+  /* Request the list of themes. If it wasn't updated yet, the "changed"
+   * callback will build the card themes submenu.
+   */
+  games_card_themes_request_themes (priv->theme_manager);
 }
 
 /* Callbacks */
@@ -1564,7 +1696,7 @@ sync_game_score (AisleriotGame *game,
    * then translate this string to "%I6d", else to "%6d".
    * Do not translate it to anything else!
    */
-  g_snprintf (str, sizeof (str), Q_("score|%6d"), score);
+  g_snprintf (str, sizeof (str), C_("score", "%6d"), score);
   gtk_label_set_text (GTK_LABEL (priv->score_label), str);
 }
 
@@ -1851,6 +1983,52 @@ game_exception_cb (AisleriotGame *game,
   gtk_widget_show (dialog);
 }
 
+#ifdef HAVE_CLUTTER
+
+static void
+settings_changed_cb (GtkSettings *settings,
+                     GParamSpec *pspec,
+                     AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+  GtkAction *action;
+  gboolean enabled;
+
+  g_object_get (settings, "gtk-enable-animations", &enabled, NULL);
+
+  action = gtk_action_group_get_action (priv->action_group, "Animations");
+  gtk_action_set_visible (action, enabled);
+}
+
+static void
+screen_changed_cb (GtkWidget *widget,
+                   GdkScreen *previous_screen,
+                   AisleriotWindow *window)
+{
+  GdkScreen *screen;
+  GtkSettings *settings;
+
+  screen = gtk_widget_get_screen (widget);
+  if (screen == previous_screen)
+    return;
+
+  if (previous_screen) {
+    g_signal_handlers_disconnect_by_func (gtk_settings_get_for_screen (previous_screen),
+                                          G_CALLBACK (settings_changed_cb),
+                                          window);
+  }
+  
+  if (screen == NULL)
+    return;
+
+  settings = gtk_widget_get_settings (widget);
+  settings_changed_cb (settings, NULL, window);
+  g_signal_connect (settings, "notify::gtk-enable-animations",
+                    G_CALLBACK (settings_changed_cb), window);
+}
+  
+#endif /* HAVE_CLUTTER */
+
 /* Class implementation */
 
 #ifdef HAVE_HILDON
@@ -1859,6 +2037,32 @@ G_DEFINE_TYPE (AisleriotWindow, aisleriot_window, HILDON_TYPE_WINDOW);
 G_DEFINE_TYPE (AisleriotWindow, aisleriot_window, GTK_TYPE_WINDOW);
 #endif
 
+#if GTK_CHECK_VERSION (2, 10, 0)
+
+static void
+aisleriot_window_style_set (GtkWidget *widget,
+                            GtkStyle *previous_style)
+{
+  AisleriotWindow *window = AISLERIOT_WINDOW (widget);
+  AisleriotWindowPrivate *priv = window->priv;
+  const cairo_font_options_t *font_options;
+  void (* style_set) (GtkWidget *, GtkStyle *) =
+    GTK_WIDGET_CLASS (aisleriot_window_parent_class)->style_set;
+
+  if (style_set)
+    style_set (widget, previous_style);
+
+  if (!priv->theme)
+    return;
+
+  font_options = gdk_screen_get_font_options (gtk_widget_get_screen (widget));
+  games_card_theme_set_font_options (priv->theme, font_options);
+
+  /* FIXMEchpe: clear the cached cards in the slots?? */
+}
+
+#endif /* GTK >= 2.10.0 */
+
 static gboolean
 aisleriot_window_state_event (GtkWidget *widget,
                               GdkEventWindowState *event)
@@ -1970,6 +2174,11 @@ aisleriot_window_init (AisleriotWindow *window)
     { "About", GTK_STOCK_ABOUT, NULL, NULL,
       ACTION_TOOLTIP (N_("About this game")),
       G_CALLBACK (help_about_cb) },
+#ifdef ENABLE_CARD_THEMES_INSTALLER
+    { "InstallThemes", NULL, N_("Install card themes…"), NULL,
+      ACTION_TOOLTIP (N_("Install new card themes from the distribution packages repositories")),
+      G_CALLBACK (install_themes_cb) },
+#endif /* ENABLE_CARD_THEMES_INSTALLER */
 
     /* Toolbar-only actions */
 #ifndef HAVE_MAEMO
@@ -2030,10 +2239,16 @@ aisleriot_window_init (AisleriotWindow *window)
       ACTION_TOOLTIP (N_("Pick up and drop cards by clicking")),
       G_CALLBACK (clickmove_toggle_cb),
       FALSE /* not active by default */ },
-   { "Sound", NULL, N_("_Enable sounds"), NULL,
-      ACTION_TOOLTIP (N_("Whether or not to play event sounds.")),
+   { "Sound", NULL, N_("_Sound"), NULL,
+      ACTION_TOOLTIP (N_("Whether or not to play event sounds")),
       G_CALLBACK (sound_toggle_cb),
       FALSE /* not active by default */ },
+#ifdef HAVE_CLUTTER
+   { "Animations", NULL, N_("_Animations"), NULL,
+      ACTION_TOOLTIP (N_("Whether or not to animate card moves")),
+      G_CALLBACK (animations_toggle_cb),
+      FALSE /* not active by default */ },
+#endif /* HAVE_CLUTTER */
 #ifdef ENABLE_DEBUG_UI
     { "DebugPixbufDrawing", NULL, "_Pixbuf drawing", NULL, NULL,
       G_CALLBACK (debug_pixbuf_drawing_cb),
@@ -2116,7 +2331,13 @@ aisleriot_window_init (AisleriotWindow *window)
           "<menuitem action='Toolbar'/>"
           "<menuitem action='Statusbar'/>"
           "<separator/>"
-          "<menu action='ThemeMenu'/>"
+          "<menu action='ThemeMenu'>"
+            "<placeholder name='ThemesPH'/>"
+#ifdef ENABLE_CARD_THEMES_INSTALLER
+            "<separator/>"
+            "<menuitem action='InstallThemes'/>"
+#endif
+          "</menu>"
         "</menu>"
         "<menu action='ControlMenu'>"
           "<menuitem action='UndoMove'/>"
@@ -2126,6 +2347,9 @@ aisleriot_window_init (AisleriotWindow *window)
           "<separator/>"
           "<menuitem action='ClickToMove'/>"
           "<menuitem action='Sound'/>"
+#ifdef HAVE_CLUTTER
+          "<menuitem action='Animations'/>"
+#endif
         "</menu>"
         "<menu action='OptionsMenu'/>"
         "<menu action='HelpMenu'>"
@@ -2183,7 +2407,8 @@ aisleriot_window_init (AisleriotWindow *window)
   GtkWidget *main_vbox;
   GtkAccelGroup *accel_group;
   GtkAction *action;
-  char *theme;
+  char *theme_name;
+  GamesCardTheme *theme;
   guint i;
 #ifdef HAVE_HILDON
   GtkToolItem *tool_item;
@@ -2199,57 +2424,41 @@ aisleriot_window_init (AisleriotWindow *window)
 
   priv->game = aisleriot_game_new ();
 
-  /* FIXMEchpe: case HAVE_HILDON && HAVE_RSVG ! */
 #ifdef HAVE_HILDON
-  priv->scalable_cards = FALSE;
   priv->use_pixbuf_drawing = FALSE;
 #else
-#ifdef HAVE_RSVG
-  /* Default to scalable */
-  env = g_getenv ("AISLERIOT_CARDS_SCALABLE");
-  priv->scalable_cards = env == NULL || g_ascii_strtoull (env, NULL, 0) != 0;
-#else
-  priv->scalable_cards = FALSE;
-#endif /* HAVE_RSVG */
-
   /* Default to pixbuf drawing */
   env = g_getenv ("AISLERIOT_PIXBUF_DRAWING");
   priv->use_pixbuf_drawing = env == NULL || g_ascii_strtoull (env, NULL, 10) != 0;
+#endif /* HAVE_HILDON */
 
 #ifdef GNOME_ENABLE_DEBUG
-  if (priv->scalable_cards)
-    g_print ("Using scalable cards\n");
-  else
-    g_print ("Using prerendered card images\n");
-
   if (priv->use_pixbuf_drawing)
     g_print ("Using pixbuf drawing method\n");
   else
     g_print ("Using pixmap drawing method\n");
 #endif /* GNOME_ENABLE_DEBUG */
 
-#endif /* HAVE_MAEMO */
+  priv->theme_manager = games_card_themes_new ();
 
-  priv->board = AISLERIOT_BOARD (aisleriot_board_new (priv->game, priv->scalable_cards));
+  priv->board = AISLERIOT_BOARD (aisleriot_board_new (priv->game));
 
   aisleriot_board_set_pixbuf_drawing (priv->board, priv->use_pixbuf_drawing);
 
-  theme = games_conf_get_string (NULL, aisleriot_conf_get_key (CONF_THEME), NULL);
-  if (!theme || !theme[0]) {
-    g_free (theme);
-    theme = g_strdup (GAMES_CARD_THEME_DEFAULT);
+  theme_name = games_conf_get_string (NULL, aisleriot_conf_get_key (CONF_THEME), NULL);
+  theme = games_card_themes_get_theme_by_name (priv->theme_manager, theme_name);
+  g_free (theme_name);
+  if (!theme) {
+    /* Last-ditch fallback: try getting *any* theme */
+    theme = games_card_themes_get_theme_any (priv->theme_manager);
   }
-#ifdef HAVE_GNOME
-  /* Compatibility with old settings */
-  if (g_str_has_suffix (theme, ".svg")) {
-    *g_strrstr (theme, ".svg") = '\0';
-  } else if (g_str_has_suffix (theme, ".png")) {
-    *g_strrstr (theme, ".png") = '\0';
+  if (theme) {
+    aisleriot_window_take_card_theme (window, theme /* adopts */);
+  } else {
+    /* FIXMEchpe: FUCK, what now? Panic! */
+    /* Put up some UI, and exit! */
+    g_assert_not_reached ();
   }
-#endif
-
-  aisleriot_board_set_card_theme (priv->board, theme);
-  g_free (theme);
 
   priv->action_group = gtk_action_group_new ("MenuActions");
   gtk_action_group_set_translation_domain (priv->action_group, GETTEXT_PACKAGE);
@@ -2352,10 +2561,40 @@ aisleriot_window_init (AisleriotWindow *window)
   gtk_widget_show_all (GTK_WIDGET (tool_item));
 #endif
 
-  /* The actions and menus are done. Add the card themes menu; the
+  /* Defer building the card themes menu until its parent's menu is opened */
+#ifdef HAVE_HILDON
+  /* FIXMEchpe check this works! */
+  g_signal_connect (priv->main_menu, "show",
+                    G_CALLBACK (main_menu_show_cb), window);
+#else
+  action = gtk_action_group_get_action (priv->action_group, "ViewMenu");
+  g_signal_connect (action, "activate",
+                    G_CALLBACK (view_menu_activate_cb), window);
+
+  /* FIXMEchpe: this isn't right. We do want to hide the menu if there's no
+   * entry in it after the list is updated...
+   */
+#if 0
+  /* So the menu doesn't change size when the theme submenu is installed */
+  action = gtk_action_group_get_action (priv->action_group, "ThemeMenu");
+  g_object_set (action, "hide-if-empty", GINT_TO_POINTER (FALSE), NULL);
+#endif
+#endif
+
+  /* It's possible that the themes list has already been loaded (e.g.
+   * if the theme loading above involved the fallback); in that case
+   * we need to update the menu right now.
+   */
+  if (games_card_themes_get_themes_loaded (priv->theme_manager))
+    install_card_theme_menu (priv->theme_manager, window);
+
+  /* Rebuild the themes menu when the themes list changes */
+  g_signal_connect (priv->theme_manager, "changed",
+                    G_CALLBACK (install_card_theme_menu), window);
+
+  /* The actions and menus are done. The
    * recent games menu will be updated when the initial game loads.
    */
-  install_card_theme_menu (window);
 
   accel_group = gtk_ui_manager_get_accel_group (priv->ui_manager);
   gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
@@ -2378,8 +2617,22 @@ aisleriot_window_init (AisleriotWindow *window)
   action = gtk_action_group_get_action (priv->action_group, "RecentMenu");
   g_object_set (action, "hide-if-empty", FALSE, NULL);
 
+#ifdef ENABLE_CARD_THEMES_INSTALLER
+  action = gtk_action_group_get_action (priv->action_group, "InstallThemes");
+  gtk_action_set_sensitive (action, games_card_themes_can_install_themes (priv->theme_manager));
+#endif /* ENABLE_CARD_THEMES_INSTALLER */
+
   set_fullscreen_actions (window, FALSE);
 
+#ifdef HAVE_CLUTTER
+  action = gtk_action_group_get_action (priv->action_group, "Animations");
+  gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+                                games_conf_get_boolean (NULL, aisleriot_conf_get_key (CONF_ANIMATIONS), NULL));
+
+  /* Set the action visibility and listen for animation mode changes */
+  screen_changed_cb (GTK_WIDGET (window), NULL, window);
+#endif /* HAVE_CLUTTER */
+
   /* Now set up the widgets */
   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), main_vbox);
@@ -2464,6 +2717,12 @@ aisleriot_window_finalize (GObject *object)
   AisleriotWindow *window = AISLERIOT_WINDOW (object);
   AisleriotWindowPrivate *priv = window->priv;
 
+#ifdef HAVE_CLUTTER
+  g_signal_handlers_disconnect_by_func (gtk_widget_get_settings (GTK_WIDGET (window)),
+                                        G_CALLBACK (settings_changed_cb),
+                                        window);
+#endif /* HAVE_CLUTTER */
+
 #ifndef HAVE_HILDON
   if (priv->hint_dialog) {
     gtk_widget_destroy (priv->hint_dialog);
@@ -2483,6 +2742,12 @@ aisleriot_window_finalize (GObject *object)
     g_assert (priv->stats_dialog == NULL);
   }
 
+  if (priv->theme) {
+    g_object_unref (priv->theme);
+  }
+
+  g_object_unref (priv->theme_manager);
+
   g_signal_handlers_disconnect_matched (priv->game,
                                         G_SIGNAL_MATCH_DATA,
                                         0, 0, NULL, NULL, window);
@@ -2504,6 +2769,9 @@ aisleriot_window_class_init (AisleriotWindowClass *klass)
   gobject_class->finalize = aisleriot_window_finalize;
 
   widget_class->window_state_event = aisleriot_window_state_event;
+#if GTK_CHECK_VERSION (2, 10, 0)
+  widget_class->style_set = aisleriot_window_style_set;
+#endif
 
   g_type_class_add_private (gobject_class, sizeof (AisleriotWindowPrivate));
 }
@@ -2573,7 +2841,7 @@ load_idle_cb (LoadIdleData *data)
     GtkWidget *dialog;
     char *name;
 
-    name = aisleriot_util_get_display_filename (data->game_file);
+    name = games_filename_to_display_name (data->game_file);
 
     dialog = gtk_message_dialog_new (GTK_WINDOW (data->window),
                                      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
