@@ -20,16 +20,22 @@
  */
 
 #include <config.h>
-#include <gnome.h>
-#include <string.h>
-#include <gdk/gdkkeysyms.h>
-#include <time.h>
 
-#include <games-gridframe.h>
-#include <games-stock.h>
-#include <games-scores.h>
-#include <games-sound.h>
-#include <games-conf.h>
+#include <string.h>
+#include <time.h>
+#include <stdlib.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
+
+#include <libgames-support/games-conf.h>
+#include <libgames-support/games-gridframe.h>
+#include <libgames-support/games-help.h>
+#include <libgames-support/games-runtime.h>
+#include <libgames-support/games-scores.h>
+#include <libgames-support/games-sound.h>
+#include <libgames-support/games-stock.h>
 
 #include "main.h"
 #include "properties.h"
@@ -42,8 +48,8 @@
 #include "warp.h"
 
 #ifdef GGZ_CLIENT
-#include <games-dlg-chat.h>
-#include <games-dlg-players.h>
+#include <libgames-support/games-dlg-chat.h>
+#include <libgames-support/games-dlg-players.h>
 #include "ggz-network.h"
 #include <ggz-embed.h>
 #endif
@@ -53,24 +59,19 @@
 
 GtkWidget *window;
 GtkWidget *drawing_area;
-GtkWidget *appbar;
+GtkWidget *statusbar;
 GtkWidget *notebook;
 GtkWidget *chat = NULL;
 
-static const GamesScoresCategory scorecats[] = { {"4.0", N_("Beginner")},
-{"3.0", N_("Slow")},
-{"2.0", N_("gnibbles|Medium")},
-{"1.0", N_("Fast")},
-{"4.1", N_("Beginner with Fakes")},
-{"3.1", N_("Slow with Fakes")},
-{"2.1", N_("Medium with Fakes")},
-{"1.1", N_("Fast with Fakes")},
-GAMES_SCORES_LAST_CATEGORY
-};
-static const GamesScoresDescription scoredesc = { scorecats,
-  "4.0",
-  "gnibbles",
-  GAMES_SCORES_STYLE_PLAIN_DESCENDING
+static const GamesScoresCategory scorecats[] = {
+{ "4.0", NC_("game speed", "Beginner")            },
+{ "3.0", NC_("game speed", "Slow")                },
+{ "2.0", NC_("game speed", "Medium")              },
+{ "1.0", NC_("game speed", "Fast")                },
+{ "4.1", NC_("game speed", "Beginner with Fakes") },
+{ "3.1", NC_("game speed", "Slow with Fakes")     },
+{ "2.1", NC_("game speed", "Medium with Fakes")   },
+{ "1.1", NC_("game speed", "Fast with Fakes")     }
 };
 
 GamesScores *highscores;
@@ -250,7 +251,7 @@ about_cb (GtkAction * action, gpointer data)
 #endif
 			 "version", VERSION,
 			 "copyright",
-			 "Copyright \xc2\xa9 1999-2007 Sean MacIsaac, Ian Peters, Andreas Røsdal",
+			 "Copyright \xc2\xa9 1999-2008 Sean MacIsaac, Ian Peters, Andreas Røsdal",
 			 "license", license, "comments",
 			 _("A worm game for GNOME.\n\nNibbles is a part of GNOME Games."), "authors", authors,
 			 "documenters", documenters, "translator-credits",
@@ -468,7 +469,7 @@ new_game (void)
   if (dummy_id)
     g_source_remove (dummy_id);
 
-  dummy_id = g_timeout_add (1500, (GSourceFunc) new_game_2_cb, NULL);
+  dummy_id = g_timeout_add_seconds (1, (GSourceFunc) new_game_2_cb, NULL);
 
   network_gui_update ();
 
@@ -491,8 +492,8 @@ pause_game_cb (GtkAction * action, gpointer data)
     /*
      * main_id = gtk_timeout_add (GAMEDELAY * properties->gamespeed,
      * (GtkFunction) main_loop, NULL);
-     * keyboard_id = gtk_signal_connect (GTK_OBJECT (window),
-     * "key_press_event", GTK_SIGNAL_FUNC (key_press_cb),
+     * keyboard_id = g_signal_connect (GTK_OBJECT (window),
+     * "key_press_event", G_CALLBACK (key_press_cb),
      * NULL);
      * add_bonus_id = gtk_timeout_add (BONUSDELAY *
      * properties->gamespeed,
@@ -608,7 +609,7 @@ restart_game (gpointer data)
 
   gnibbles_add_bonus (1);
 
-  dummy_id = g_timeout_add (1500, (GSourceFunc) new_game_2_cb, NULL);
+  dummy_id = g_timeout_add_seconds (1, (GSourceFunc) new_game_2_cb, NULL);
 
   restart_id = 0;
 
@@ -669,7 +670,7 @@ main_loop (gpointer data)
       g_source_remove (add_bonus_id);
     }
     add_bonus_id = 0;
-    erase_id = g_timeout_add (3000,
+    erase_id = g_timeout_add_seconds (3,
 			      (GSourceFunc) erase_worms_cb,
 			      (gpointer) ERASESIZE);
     gnibbles_log_score (window);
@@ -690,7 +691,7 @@ main_loop (gpointer data)
       g_source_remove (add_bonus_id);
     }
     add_bonus_id = 0;
-    erase_id = g_timeout_add (3000,
+    erase_id = g_timeout_add_seconds (3,
 			      (GSourceFunc) erase_worms_cb,
 			      (gpointer) ERASESIZE);
     gnibbles_log_score (window);
@@ -721,7 +722,7 @@ main_loop (gpointer data)
     erase_id = g_timeout_add (ERASETIME / ERASESIZE,
 			      (GSourceFunc) erase_worms_cb,
 			      (gpointer) ERASESIZE);
-    restart_id = g_timeout_add (1000, (GSourceFunc) restart_game, NULL);
+    restart_id = g_timeout_add_seconds (1, (GSourceFunc) restart_game, NULL);
     return (FALSE);
   }
 
@@ -750,7 +751,7 @@ main_loop (gpointer data)
 	tmp = rand () % MAXLEVEL + 1;
       current_level = tmp;
     }
-    restart_id = g_timeout_add (1000, (GSourceFunc) restart_game, NULL);
+    restart_id = g_timeout_add_seconds (1, (GSourceFunc) restart_game, NULL);
     return (FALSE);
   }
 
@@ -767,7 +768,7 @@ show_cursor_cb (GtkWidget * widget, GdkEventMotion * event, gpointer data)
 static void
 help_cb (GtkAction * action, gpointer data)
 {
-  gnome_help_display ("gnibbles.xml", NULL, NULL);
+  games_help_display (window, "gnibbles", NULL);
 }
 
 static const GtkActionEntry action_entry[] = {
@@ -872,12 +873,14 @@ setup_window (void)
 {
   GdkPixmap *cursor_dot_pm;
   GtkWidget *vbox;
+  GtkWidget *main_vbox;
   GtkWidget *packing;
   GtkWidget *menubar;
   GtkUIManager *ui_manager;
   GtkAccelGroup *accel_group;
 
-  window = gnome_app_new ("gnibbles", "Nibbles");
+  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title (GTK_WINDOW (window), _("Nibbles"));
 
   gtk_window_set_default_size (GTK_WINDOW (window), DEFAULT_WIDTH, DEFAULT_HEIGHT);
   games_conf_add_window (GTK_WINDOW (window), KEY_PREFERENCES_GROUP);
@@ -947,20 +950,24 @@ setup_window (void)
   gtk_widget_set_events (drawing_area, GDK_BUTTON_PRESS_MASK |
 			 GDK_EXPOSURE_MASK | GDK_POINTER_MOTION_MASK);
 
-  gnome_app_set_contents (GNOME_APP (window), notebook);
+  main_vbox = gtk_vbox_new (FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (main_vbox), notebook, TRUE, TRUE, 0);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, NULL);
   gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), MAIN_PAGE);
 
+  statusbar = gtk_statusbar_new ();
+  gtk_box_pack_start (GTK_BOX (main_vbox), statusbar, FALSE, FALSE, 0);
+
+  gtk_container_add (GTK_CONTAINER (window), main_vbox);
+
+
   gtk_widget_show_all (window);
 #ifdef GGZ_CLIENT
   gtk_widget_hide (chat);
 #endif
   gtk_widget_show (drawing_area);
 
-  appbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
-  gnome_app_set_statusbar (GNOME_APP (window), appbar);
-
-  scoreboard = gnibbles_scoreboard_new (appbar);
+  scoreboard = gnibbles_scoreboard_new (statusbar);
 
 }
 
@@ -1028,28 +1035,42 @@ render_logo (void)
 int
 main (int argc, char **argv)
 {
-  GnomeProgram *program;
   GOptionContext *context;
+  gboolean retval;
+  GError *error = NULL;
+
+  if (!games_runtime_init ("gnibbles"))
+    return 1;
 
   setgid_io_init ();
 
-  bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-  textdomain (GETTEXT_PACKAGE);
-
-  g_thread_init (NULL);
   context = g_option_context_new (NULL);
+
+#if GLIB_CHECK_VERSION (2, 12, 0)
+  g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
+#endif 
+
+  g_option_context_add_group (context, gtk_get_option_group (TRUE));
   games_sound_add_option_group (context);
 
-  program = gnome_program_init ("gnibbles", VERSION, LIBGNOMEUI_MODULE,
-				argc, argv,
-				GNOME_PARAM_GOPTION_CONTEXT, context,
-				GNOME_PARAM_POPT_TABLE, NULL,
-				GNOME_PARAM_APP_DATADIR, REAL_DATADIR, NULL);
+  retval = g_option_context_parse (context, &argc, &argv, &error);
+  g_option_context_free (context);
+  if (!retval) {
+    g_print ("%s", error->message);
+    g_error_free (error);
+    exit (1);
+  }
+
+  g_set_application_name (_("Nibbles"));
+
   gtk_window_set_default_icon_name ("gnome-gnibbles");
   srand (time (NULL));
 
-  highscores = games_scores_new (&scoredesc);
+  highscores = games_scores_new ("gnibbles",
+                                 scorecats, G_N_ELEMENTS (scorecats),
+                                 "game speed", NULL,
+                                 0 /* default category */,
+                                 GAMES_SCORES_STYLE_PLAIN_DESCENDING);
 
   games_conf_initialise ("Gnibbles");
 
@@ -1085,7 +1106,7 @@ main (int argc, char **argv)
 
   games_conf_shutdown ();
 
-  g_object_unref (program);
+  games_runtime_shutdown ();
 
   return 0;
 }
