@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import gtk, gobject, os.path
 import re
 import xml.sax.saxutils
@@ -410,7 +411,6 @@ class BooleanDialog (MessageDialog):
         else:
             self.hide()
             if self.modal: gtk.main_quit()
-            raise UserCancelledError("%s Cancelled"%self)
 
     def nocb (self, *args):
         self.ret=False
