@@ -21,6 +21,9 @@
  * For more details see the file COPYING.
  */
 
+#include <config.h>
+
+#include <string.h>
 
 #include "renderer.h"
 
